@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
 # this work for additional information regarding copyright ownership.
@@ -114,8 +115,7 @@ sub status_rlimit {
                   "</tr>");
 
     for my $res (keys %$lim) {
-        my $val = eval "&BSD::Resource::${res}()";
-        my ($soft, $hard) = getrlimit $val;
+        my ($soft, $hard) = getrlimit($lim->{$res});
         (my $limit = $res) =~ s/^RLIMIT_//;
         ($soft, $hard) = ("$soft " . BM($soft), "$hard ". BM($hard))
             if $is_mb{$limit};
