@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
 # this work for additional information regarding copyright ownership.
@@ -141,7 +142,7 @@ sub handler {
 sub header {
     my $r = shift;
     my $start = scalar localtime $^T;
-    my $srv = Apache2::ServerUtil::get_server_version();
+    my $srv = Apache2::ServerUtil::get_server_banner();
     $r->content_type("text/html");
     my $v = $^V ? sprintf "v%vd", $^V : $];
     $r->print(<<"EOF");
