@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
 # this work for additional information regarding copyright ownership.
@@ -61,7 +62,9 @@ use mod_perl2 ();
 use Symbol ();
 use File::Spec ();
 
-use APR::Const -compile => qw(FINFO_NORM);
+use APR::Const -compile => qw(FINFO_NORM FINFO_PROT);
+
+use constant WIN32 => ($^O eq "MSWin32");
 
 BEGIN {
     $INC{'Apache.pm'} = __FILE__;
@@ -91,7 +94,11 @@ my %overridable_mp2_api = (
         if (defined $newfile) {
             $old_filename = $r->$orig_sub($newfile);
             die "'$newfile' doesn't exist" unless -e $newfile;
-            $r->finfo(APR::Finfo::stat($newfile, APR::Const::FINFO_NORM, $r->pool));
+            my $wanted = APR::Const::FINFO_NORM;
+            if (WIN32) {
+                $wanted &= ~APR::Const::FINFO_PROT;
+            }
+            $r->finfo(APR::Finfo::stat($newfile, $wanted, $r->pool));
         }
         else {
             $old_filename = $r->$orig_sub();
@@ -149,7 +156,12 @@ EOI
     require Apache2::Connection;
     require APR::SockAddr;
     require Socket;
-    my $orig_sub = *Apache2::Connection::remote_addr{CODE};
+    my $orig_sub;
+    if (defined *Apache2::Connection::client_addr{CODE}) { # httpd-2.4
+        $orig_sub = *Apache2::Connection::client_addr{CODE};
+    } else { # httpd-2.2
+        $orig_sub = *Apache2::Connection::remote_addr{CODE};
+    }
     *Apache2::Connection::remote_addr = sub {
         my $c = shift;
         if (@_) {
@@ -246,6 +258,9 @@ sub override_mp2_api {
             next;
         }
         $overridden_mp2_api{$sub} = eval $overridable_mp2_api{$sub};
+        if ($@) {
+            die "error overriding $sub : $@";
+        }
         unless (exists $overridden_mp2_api{$sub} &&
                 ref($overridden_mp2_api{$sub}) eq 'CODE') {
             die "overriding $sub didn't return a CODE ref";
