@@ -42,6 +42,11 @@ MP_CMD_SRV_DECLARE(modules);
 MP_CMD_SRV_DECLARE(requires);
 MP_CMD_SRV_DECLARE(config_requires);
 MP_CMD_SRV_DECLARE(post_config_requires);
+#if AP_SERVER_MAJORVERSION_NUMBER>2 || \
+    (AP_SERVER_MAJORVERSION_NUMBER == 2 && AP_SERVER_MINORVERSION_NUMBER>=3)
+MP_CMD_SRV_DECLARE2(authz_provider);
+MP_CMD_SRV_DECLARE2(authn_provider);
+#endif
 MP_CMD_SRV_DECLARE2(set_var);
 MP_CMD_SRV_DECLARE2(add_var);
 MP_CMD_SRV_DECLARE2(set_env);
@@ -72,22 +77,6 @@ MP_CMD_SRV_DECLARE(interp_max);
 MP_CMD_SRV_DECLARE(interp_max_spare);
 MP_CMD_SRV_DECLARE(interp_min_spare);
 MP_CMD_SRV_DECLARE(interp_max_requests);
-MP_CMD_SRV_DECLARE(interp_scope);
-
-#define modperl_interp_scope_undef(dcfg) \
-(dcfg->interp_scope == MP_INTERP_SCOPE_UNDEF)
-
-#define modperl_interp_scope_handler(dcfg) \
-(dcfg->interp_scope == MP_INTERP_SCOPE_HANDLER)
-
-#define modperl_interp_scope_subrequest(dcfg) \
-(dcfg->interp_scope == MP_INTERP_SCOPE_SUBREQUEST)
-
-#define modperl_interp_scope_request(scfg) \
-(scfg->interp_scope == MP_INTERP_SCOPE_REQUEST)
-
-#define modperl_interp_scope_connection(scfg) \
-(scfg->interp_scope == MP_INTERP_SCOPE_CONNECTION)
 
 #endif /* USE_ITHREADS */
 
@@ -115,6 +104,10 @@ MP_CMD_SRV_DECLARE(interp_scope);
    AP_INIT_ITERATE( name, modperl_cmd_##item, NULL, \
       RSRC_CONF, desc )
 
+#define MP_CMD_SRV_ITERATE_ON_READ(name, item, desc) \
+   AP_INIT_ITERATE( name, modperl_cmd_##item, NULL, \
+      RSRC_CONF|EXEC_ON_READ, desc )
+
 #define MP_CMD_SRV_ITERATE2(name, item, desc) \
    AP_INIT_ITERATE2( name, modperl_cmd_##item, NULL, \
       RSRC_CONF, desc )
@@ -148,3 +141,10 @@ MP_CMD_SRV_DECLARE(interp_scope);
       OR_ALL|EXEC_ON_READ, desc )
 
 #endif /* MODPERL_CMD_H */
+
+/*
+ * Local Variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ */
