@@ -79,7 +79,7 @@
 /* avoiding -Wall warning */
 
 #undef dNOOP
-#define dNOOP extern int __attribute__ ((unused)) Perl___notused
+#define dNOOP extern int __attribute__ ((unused)) Perl___notused___modperl
 
 #ifndef G_METHOD
 #   define G_METHOD 64
@@ -139,3 +139,10 @@
 #endif
 
 #endif /* MODPERL_PERL_INCLUDES_H */
+
+/*
+ * Local Variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ */
