@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 package TestHooks::authen_digest;
 
 use strict;
@@ -21,6 +22,9 @@ sub handler {
         return Apache2::Const::HTTP_UNAUTHORIZED;
     }
 
+    $r->user("user");
+    $r->ap_auth_type("Digest");
+
     return Apache2::Const::OK;
 }
 
