@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 package TestAPI::aplog;
 
 use strict;
@@ -17,6 +18,7 @@ use TestCommon::LogDiff;
 
 use Apache2::Const -compile => qw(OK :log);
 use APR::Const    -compile => qw(:error SUCCESS);
+use constant APACHE24   => have_min_apache_version('2.4.0');
 
 my @LogLevels = qw(emerg alert crit error warn notice info debug);
 my $package = __PACKAGE__;
@@ -59,6 +61,11 @@ sub handler {
 
     # log_serror
     {
+        my $orig_log_level = 0;
+        if (APACHE24) {
+            $orig_log_level = $s->loglevel;
+            $s->loglevel(Apache2::Const::LOG_DEBUG);
+        }
         t_server_log_warn_is_expected();
         $s->log_serror(Apache2::Log::LOG_MARK,
                        Apache2::Const::LOG_INFO|Apache2::Const::LOG_STARTUP,
@@ -76,7 +83,7 @@ sub handler {
 
         # the APR_EGENERAL error string changed for APR 1.0
         my $egeneral = have_min_apache_version('2.1.0')
-           ? "Internal error"
+           ? qr/Internal error(?: \(specific information not available\))?/
            : "Error string not specified yet";
 
         t_server_log_warn_is_expected();
@@ -85,6 +92,9 @@ sub handler {
         ok t_cmp $logdiff->diff,
             qr/$egeneral: log_serror test 2/,
             '$s->log_serror(LOG_MARK, LOG_DEBUG, APR::Const::EGENERAL...)';
+        if (APACHE24) {
+            $s->loglevel($orig_log_level);
+        }
     }
 
     # log_rerror
@@ -92,38 +102,73 @@ sub handler {
     $r->log_rerror(Apache2::Log::LOG_MARK, Apache2::Const::LOG_CRIT,
                    APR::Const::ENOTIME, "log_rerror test");
     # can't match against the error string, since a locale may kick in
-    ok t_cmp $logdiff->diff,
-        qr/\[crit\] .*?: log_rerror test/,
-        '$r->log_rerror(LOG_MARK, LOG_CRIT, APR::Const::ENOTIME...)';
+    if (APACHE24) {
+        ok t_cmp $logdiff->diff,
+            qr/\[\w*:crit\] \[pid[^]]+\] .*?: \[[^]]+\] log_rerror test/,
+            '$r->log_rerror(LOG_MARK, LOG_CRIT, APR::Const::ENOTIME...)';
+    }
+    else {
+        ok t_cmp $logdiff->diff,
+            qr/\[crit\] .*?: log_rerror test/,
+            '$r->log_rerror(LOG_MARK, LOG_CRIT, APR::Const::ENOTIME...)';
+    }
 
     # log_error
     {
         t_server_log_error_is_expected();
         $r->log_error('$r->log_error test');
-        ok t_cmp $logdiff->diff,
-            qr/\[error\] \$r->log_error test/,
-            '$r->log_error(...)';
+        if (APACHE24) {
+            ok t_cmp $logdiff->diff,
+                qr/\[\w*:error\] \[pid[^]]+\] \$r->log_error test/,
+                '$r->log_error(...)';
+        }
+        else {
+            ok t_cmp $logdiff->diff,
+                qr/\[error\] \$r->log_error test/,
+                '$r->log_error(...)';
+        }
 
         t_server_log_error_is_expected();
         $s->log_error('$s->log_error test');
-        ok t_cmp $logdiff->diff,
-            qr/\[error\] \$s->log_error test/,
-            '$s->log_error(...)';
+        if (APACHE24) {
+            ok t_cmp $logdiff->diff,
+                qr/\[\w*:error\] \[pid[^]]+\] \$s->log_error test/,
+                '$s->log_error(...)';
+        }
+        else {
+            ok t_cmp $logdiff->diff,
+                qr/\[error\] \$s->log_error test/,
+                '$s->log_error(...)';
+        }
     }
 
     # log_reason
     {
         t_server_log_error_is_expected();
         $r->log_reason('$r->log_reason test');
-        ok t_cmp $logdiff->diff,
-            qr/\[error\] access to.*failed.*reason: \$r->log_reason test/,
-            '$r->log_reason(msg)';
+        if (APACHE24) {
+            ok t_cmp $logdiff->diff,
+                qr/\[\w*:error\] \[pid[^]]+\] access to.*failed.*reason: \$r->log_reason test/,
+                '$r->log_reason(msg)';
+        }
+        else {
+            ok t_cmp $logdiff->diff,
+                qr/\[error\] access to.*failed.*reason: \$r->log_reason test/,
+                '$r->log_reason(msg)';
+        }
 
         t_server_log_error_is_expected();
         $r->log_reason('$r->log_reason filename test','filename');
-        ok t_cmp $logdiff->diff,
-            qr/\[error\] access to filename failed.*\$r->log_reason filename test/,
-            '$r->log_reason(msg, filename)';
+        if (APACHE24) {
+            ok t_cmp $logdiff->diff,
+                qr/\[\w*:error\] \[pid[^]]+\] access to filename failed.*\$r->log_reason filename test/,
+                '$r->log_reason(msg, filename)';
+        }
+        else {
+            ok t_cmp $logdiff->diff,
+                qr/\[error\] access to filename failed.*\$r->log_reason filename test/,
+                '$r->log_reason(msg, filename)';
+        }
     }
 
     # XXX: at the moment we can't change loglevel after server startup
@@ -176,32 +221,60 @@ sub handler {
 
     t_server_log_warn_is_expected();
     $s->warn('$s->warn test');
-    ok t_cmp $logdiff->diff,
-        qr/\[warn\] \$s->warn test/,
-        '$s->warn()';
+    if (APACHE24) {
+        ok t_cmp $logdiff->diff,
+            qr/\[\w*:warn\] \[pid[^]]+\] \$s->warn test/,
+            '$s->warn()';
+    }
+    else {
+        ok t_cmp $logdiff->diff,
+            qr/\[warn\] \$s->warn test/,
+            '$s->warn()';
+    }
 
     {
         t_server_log_warn_is_expected();
         # this uses global server to get $s internally
         Apache2::ServerRec::warn("Apache2::ServerRec::warn test");
-        ok t_cmp $logdiff->diff,
-            qr/\[warn\] Apache2::ServerRec::warn test/,
-            'Apache2::ServerRec::warn() w/o Apache2::RequestUtil->request ';
+        if (APACHE24) {
+            ok t_cmp $logdiff->diff,
+                qr/\[\w*:warn\] \[pid[^]]+\] Apache2::ServerRec::warn test/,
+                'Apache2::ServerRec::warn() w/o Apache2::RequestUtil->request ';
+        }
+        else {
+            ok t_cmp $logdiff->diff,
+                qr/\[warn\] Apache2::ServerRec::warn test/,
+                'Apache2::ServerRec::warn() w/o Apache2::RequestUtil->request ';
+        }
 
         Apache2::RequestUtil->request($r);
         t_server_log_warn_is_expected();
         # this uses the global $r to get $s internally
         Apache2::ServerRec::warn("Apache2::ServerRec::warn test");
-        ok t_cmp $logdiff->diff,
-            qr/\[warn\] Apache2::ServerRec::warn test/,
-            'Apache2::ServerRec::warn() w/ Apache2::RequestUtil->request ';
+        if (APACHE24) {
+            ok t_cmp $logdiff->diff,
+                qr/\[\w*:warn\] \[pid[^]]+\] Apache2::ServerRec::warn test/,
+                'Apache2::ServerRec::warn() w/ Apache2::RequestUtil->request ';
+        }
+        else {
+            ok t_cmp $logdiff->diff,
+                qr/\[warn\] Apache2::ServerRec::warn test/,
+                'Apache2::ServerRec::warn() w/ Apache2::RequestUtil->request ';
+        }
     }
 
     t_server_log_warn_is_expected();
     warn "warn test";
-    ok t_cmp $logdiff->diff,
-        qr/\[warn\] warn test/,
-        'overriden via export warn()';
+    if (APACHE24) {
+        ok t_cmp $logdiff->diff,
+            qr/\[\w*:warn\] \[pid[^]]+\] warn test/,
+            'overriden via export warn()';
+    }
+    else {
+        ok t_cmp $logdiff->diff,
+            qr/\[warn\] warn test/,
+            'overriden via export warn()';
+    }
 
     Apache2::Const::OK;
 }
