@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 package TestDirective::cmdparms;
 
 use strict;
@@ -133,7 +134,7 @@ TestCmdParms "Vhost"
 </Base>
 
 TestCmdParms "Location"
-
-<LimitExcept GET>
-    TestCmdParms "Limit"
-</LimitExcept>
+#FIXME! httpd 2.4 does not allow LimitExcept here
+#<LimitExcept GET>
+    #TestCmdParms "Limit"
+#</LimitExcept>
